(function (window, document, undefined) {
  var instance = null;
  if (typeof define == 'function') {
    define('sd', ['xmls'], function (xmls) {
      return new SD(xmls);
    });
  } else {
    if (!window.xmls) {
      var xhr = new XMLHttpRequest();
      xhr.open('GET', '/~ask_crc?type="cfg"', true);
      xhr.addEventListener('load', function load() {
        if (this.readyState == 4) {
          if (this.status == 200) {
            var config = localStorage['config'];
            if (config) {
              config = JSON.parse(config);
              if (this.responseText == config.crc) {
                var parser = new DOMParser(),
                  doc = parser.parseFromString(config.cfg, 'text/xml');
                window.xmls = [doc.documentElement];
                window['sd'] = new SD(window.xmls);
              } else getXmlFromServer(this.responseText);
            } else getXmlFromServer(this.responseText);
          }
        }
      });
      xhr.send(null);
    } else window['sd'] = new SD(window.xmls);
  }

  function getXmlFromServer(crc) {
    var xhr = new XMLHttpRequest();
    xhr.open('GET', '/~cfg_ask_xml?type=cfg', true);
    xhr.addEventListener('load', function load() {
      if (this.readyState == 4) {
        if (this.status == 200) {
          if (xhr && xhr.responseText) {
            var lconf = { crc: crc, cfg: xhr.responseText };
            try {
              localStorage['config'] = JSON.stringify(lconf);
            } catch (e) {}
          }
          window.xmls = [this.responseXML.documentElement];
          window['sd'] = new SD(window.xmls);
        }
      }
    });
    xhr.send(null);
  }

  function SD(xmls) {
    if (instance) return instance;
    var self = this;

    var sizesObj = {},
      listConfig = {},
      flags = {};

    getFS();

    //чтение ListConfig
    Array.prototype.slice.call(xmls[0].querySelectorAll('ListConfig Item')).map(function (e) {
      listConfig[e.getAttribute('name')] = e.getAttribute('type');
    });

    //максимальный размер архива
    var _getArchSize = function (archNode, update) {
      var name = archNode.querySelector('Name').textContent;

      if (name in sizesObj && !update) return sizesObj[name];

      var channelData = archNode.querySelector('ChannelData').textContent,
        channelItems = xmls[0].querySelectorAll('ListOutChans OutChan[name="' + channelData + '"] Item'),
        sectionSize = 0, //размер среза
        maxEventSize = 64, //Максимальный размер события
        itemsCounter = 0, //кол-во ячеек в канале
        cfgSize, //размер файла конфигурации
        archSize; //максимальный размер архива

      var maxRecordFile = archNode.querySelector('MaxRecordFile').textContent, //Кол-во событий в файле
        maxFile = archNode.querySelector('MaxFile').textContent; //Кол-во файлов

      //Размер среза = сумма размера всех ячеек(64/128) в канале
      channelItems = Array.prototype.slice.call(channelItems);
      channelItems.map(function (item) {
        var name = item.getAttribute('name');
        var inChItem = xmls[0].querySelector('ListInChans Item[name="' + name + '"]');
        if (inChItem) {
          var cfg = inChItem.getAttribute('cfg');
          if (cfg && cfg in listConfig) {
            if (listConfig[cfg].search('BLOB') > -1) {
              //Если BLOB, то 128б
              maxEventSize = 128;
              sectionSize += 128;
            } else sectionSize += 64; //иначе 64б
          } else sectionSize += 64; //если не нашел cfg в ListConfig, то ошибка, но у меня будет по умолчанию.
        } else sectionSize += 64; //если не нашел Item в InChans, то ошибка в config и не запустится Binom, но у меня будет по умолчанию.
      });

      itemsCounter = channelItems.length;
      cfgSize = 3775 + 204 * itemsCounter;
      archSize = (sectionSize + maxRecordFile * maxEventSize) * maxFile + cfgSize;

      sizesObj[name] = Math.floor(archSize / 1024);

      return Math.floor(archSize / 1024);
    };

    self.getArchSize = function (name, update) {
      update = !(typeof update == 'undefined');
      var archNode = xmls[0].querySelectorAll('Archives Archiv');

      archNode = Array.prototype.slice.call(archNode).filter(function (e) {
        return e.querySelector('Name').textContent == name;
      });
      if (archNode.length) archNode = archNode[0];
      else return 0;

      return _getArchSize(archNode, update);
    };

    //Объект с максимальными размерами архивов и осциллограмм
    self.getSizes = function (update) {
      var fullSize = 0;

      update = !(typeof update == 'undefined');

      var archNodes = xmls[0].querySelectorAll('Archives Archiv');
      Array.prototype.slice.call(archNodes).map(function (e) {
        var path = e.querySelector('Path');
        var enable = e.querySelector('Enable');
        enable = enable !== null ? !!Number(enable.textContent) : true;
        if (path && enable) {
          path = path.textContent;
          if (!path.search('sd:')) {
            //если архив пишется на sd
            var size = _getArchSize(e, update);
            fullSize += size;
          }
        }
      });

      var oscNodes = xmls[0].querySelectorAll('Oscillographs Oscillograph');
      Array.prototype.slice.call(oscNodes).map(function (e) {
        var path = e.querySelector('Path');
        if (path) {
          path = path.textContent;
          if (!path.search('sd:')) {
            //если OSC пишется на sd
            var size = _getOSCSize(e, update);
            fullSize += size;
          }
        }
      });

      sizesObj['fullSize'] = fullSize;
      return sizesObj;
    };

    var _getOSCSize = function (oscNode, update) {
      var name = oscNode.querySelector('Name').textContent;
      if (name in sizesObj && !update) return sizesObj[name];

      var channelData = oscNode.querySelector('ChannelData,_ChannelData').textContent,
        channelItems = xmls[0].querySelectorAll('ListOutChans OutChan[name="' + channelData + '"] Item'),
        limLen, //Предельная длительность
        contCounter, //Кол-во контейнеров
        contSize = 5 + 256 * 6 * 2, //Размер контейнера
        addChanSize, //Размер доп канала
        sectionSize = 0, //размер среза
        maxEventSize = 64, //Максимальный размер события
        TMSize, //Размер TM
        cfgSize, //размер файла конфигурации
        oscCount, //Кол-во осциллограмм
        OSCSize; //Размер осциллограммы

      channelItems = Array.prototype.slice.call(channelItems);
      channelItems.map(function (item) {
        var name = item.getAttribute('name');
        var inChItem = xmls[0].querySelector('ListInChans Item[name="' + name + '"]');
        if (inChItem) {
          var cfg = inChItem.getAttribute('cfg');
          if (cfg && cfg in listConfig) {
            if (listConfig[cfg].search('BLOB') > -1) {
              //Если BLOB, то 128б
              maxEventSize = 128;
              sectionSize += 128;
            } else sectionSize += 64; //иначе 64б
          } else sectionSize += 64; //если не нашел cfg в ListConfig, то ошибка, но у меня будет по умолчанию.
        } else sectionSize += 64; //если не нашел Item в InChans, то ошибка в config и не запустится Binom, но у меня будет по умолчанию.
      });

      limLen = oscNode.querySelector('MaxTime').textContent;
      contCounter = Math.ceil(limLen / 8);
      TMSize = (9 * 100 * limLen) / 1000;
      oscCount = oscNode.querySelector('MaxRun').textContent;
      addChanSize = sectionSize + ((100 * limLen) / 1000) * maxEventSize;
      cfgSize = 3350 + 204 * channelItems.length;
      OSCSize = (contCounter * contSize + addChanSize + TMSize + cfgSize) * oscCount;

      sizesObj[name] = Math.floor(OSCSize / 1024);
      return Math.floor(OSCSize / 1024);
    };

    //максимальный размер осциллограммы
    self.getOSCSize = function (name) {
      var oscNode = xmls[0].querySelectorAll('Oscillographs Oscillograph');

      oscNode = Array.prototype.slice.call(oscNode).filter(function (e) {
        return e.querySelector('Name').textContent == name;
      });
      if (oscNode.length) oscNode = oscNode[0];
      else return 0;

      return _getArchSize(oscNode);
    };

    function getFS() {
      var xhr = new XMLHttpRequest();
      xhr.open('GET', '/~flag_system', true);
      xhr.addEventListener('load', function load() {
        if (this.readyState == 4) {
          if (this.status == 200) {
            var src = this.response;
            src = src.replace('distfree', 'diskfree');

            try {
              flags = JSON.parse(src);
              for (var i in flags) {
                if (!isNaN(Number(flags[i]))) flags[i] = Number(flags[i]);
              }
            } catch (e) {
              flags = {};
            }

            xhr.removeEventListener('load', load);
          }
        }
      });
      xhr.send(null);
      /*
		$.ajax({
			url: '/~flag_system',
			type: 'GET',
			headers: exports.requestHeaders ? exports.requestHeaders : {},
			success: function(src){
//!!!!!!!!!
				src = src.replace('distfree','diskfree');

				try {
					flags = JSON.parse(src);
					for (var i in flags) {
						if (!isNaN(Number(flags[i]))) flags[i] = Number(flags[i]);
					}
				} catch(e){
					flags = {};
				}
			},
			error: function(e, xhr, settings, exception){
				console.log('flag_system: Ошибка запроса');
			}
		})
*/
    }

    self.getFlagSystem = function () {
      return flags;
    };

    instance = this;
  }
})(window, document, undefined);
