(function(window, document, undefined) {
	var instance;

  function xhr(method, url, type, callbacks, timeout) {
    var xhr = new XMLHttpRequest();
    if ('withCredentials' in xhr) {
      xhr.open(method, url, true);
    } else if (typeof XDomainRequest != 'undefined') {
      xhr = new XDomainRequest();
      xhr.open(method, url);
    } else {
      xhr = null;
    }

    xhr.responseType = type || 'text';
    Object.keys(callbacks || {}).forEach(function(callback) {
      xhr.addEventListener.apply(xhr, [callback].concat(callbacks[callback]));
    });
    xhr.timeout = timeout;
    xhr.callbacks = callbacks;
    return xhr;
  }

  function gears (access, vars) {if (instance) return instance;
    this.access = access;
		this.vars = vars;
    this.onInit = [];
    this.init();
    instance = this;
  }
  gears.prototype.init = function () {
    this.element = document.querySelector('#gears');
    document.addEventListener('click', function(e){
      var target = e.target;
      while (target && target.tagName.toLowerCase() !== 'body') {
        if (target.id == 'gears-button') {
          this.classList.remove('exit');
          this.classList.add('enter');
          return;
        } else if (target.id == 'gears') {
          return;
        } else {
          target = target.parentNode;
        }
      }
      this.classList.add('exit');
      this.classList.remove('enter');
    }.bind(this.element));

	var lev = document.body.getAttribute('name');
    var fragment = document.createElement('div');
    var g = [
      '<div id="gears-body">',
        '<div id="overlay"></div>',
        UserData(this)
    ]
	switch(lev){
		case 'param':
			g.push(RestartForm(this));
			g.push(SaveForm(this));
			g.push(LoadForm(this));
			g.push(TimeForm(this));
			g.push(DisableHTTPForm(this));
			break;
		default:
			g.push(StationInfo(this));
	}
	g.push('</div>');
	fragment.innerHTML = g.join('');

    this.element.appendChild(fragment.children[0]);
    this.onInit.forEach(function(callback){callback()});
  }

	if (typeof define == 'function') {
		define('gears', ['ac', 'vr'],function(access, vars){
			return new gears(access, vars);
		});
	} else {
		window.gears = new gears(window.access, window.vars);
	}

  function StationInfo(gears) {
    gears.onInit.push(function(){
      var form = document.querySelector('#gears #station');
			if (this.vars) {
				fillForm(this.vars.passport);
			} else {
				var r = new xhr('GET', '/~passport', 'text/xml',
	        {
	          load: onload,
	        }
	      );
	      if (exports.requestHeaders){
	        Object.keys(exports.requestHeaders).forEach(function(key) {
	          r.setRequestHeader(key, exports.requestHeaders[key]);
	        });
	      }

				function onload() {
	        if (r.readyState == 4) {
	          if(r.status == 200){
							fillForm(r.responseXML);
	          }
	          r.onload = null;
	        }
	      };
	      r.send(null);
			}

			function fillForm(passport) {
				var params = passport.ExtraParams ? passport.ExtraParams : passport.querySelector('ExtraParams');
				if (!params) return;

				var displayForm = false;
				['StationName', 'StationVoltage', 'CellNumber', 'ConnectionName'].forEach(function(id) {
					var option = id in params ? params[id] : (params && params.querySelector) ? params.querySelector('[name="'+id+'"]') : null;
					if (!option) return;

					var value = option.getAttribute ? option.getAttribute('value') : option;
					if (!value) return;

					var display = form.querySelector('#' + id + ' .form-option-value');
					if (!display) return;

					display.innerHTML = value;
					display.parentNode.classList.remove('hidden');
					displayForm = true;
				});

				if (displayForm) form.classList.remove('hidden');
			}
      //gears.element.querySelector('#save').addEventListener('click', gears.save, false)
    }.bind(gears));
    return [
      '<div id="station" class="hidden">',
        '<div class="form-title">' + locUH.gears.info_place + '</div>',
        '<div class="form-option hidden" id="StationName"><span class="form-option-title">' + locUH.gears.info_name_substation + '</span><span class="form-option-value"></span></div>',
        '<div class="form-option hidden" id="StationVoltage"><span class="form-option-title">' + locUH.gears.info_U_class + '</span><span class="form-option-value"></span></div>',
        '<div class="form-option hidden" id="CellNumber"><span class="form-option-title">' + locUH.gears.info_num_cell + '</span><span class="form-option-value"></span></div>',
        '<div class="form-option hidden" id="ConnectionName"><span class="form-option-title">' + locUH.gears.info_name_pris + '</span><span class="form-option-value"></span></div>',
      '</div>'
    ].join('')
  }

  function UserData(gears){
    gears.onInit.push(function(){
      var name = gears.element.querySelector('#userinfo .name'),
          role = gears.element.querySelector('#userinfo .role'),
          roles = {
            operator: locUH.gears.operator,
            user: locUH.gears.user,
            custom: locUH.gears.custom,
            admin: locUH.gears.admin,
            asque: locUH.gears.asque,
            pqm: locUH.gears.pqm
          };
      name.innerHTML = gears.access.name || locUH.gears.unknown;
      role.innerHTML = roles[gears.access.role] || locUH.gears.not_defined;
    });
    return [
      '<div id="userdata">',
        '<span id="usericon"><div class="icon-user"></div></span>',
        '<span id="userinfo">',
          '<div class="name"></div>',
          '<div class="role"></div>',
          '<div class="exit"><a href="/~logout" title="' + locUH.gears.quit_title + '">' + locUH.gears.quit + '</a></div>',
        '</span>',
      '</div>'
    ].join('')
  }
  function RestartForm(gears){
    gears.onInit.push(function(){
      if (gears.access.restart) {
        gears.element.querySelector('#restart').addEventListener('click', gears.restart, false);
      }

    })
    gears.onLoad = [];
    gears.restart = function(){
      var soil = btoa,
          args = {
            password: gears.element.querySelector('#restartform_local_0').value,
            base64: true
          };

      var r = new xhr('POST', '/~restart', 'text/plain',
        {
          load: restartProgress
        }
      );
      if (exports.requestHeaders){
        Object.keys(exports.requestHeaders).forEach(function(key) {
          r.setRequestHeader(key, exports.requestHeaders[key]);
        });
      }
      r.setRequestHeader('Content-type', 'application/x-www-form-urlencoded');
      r.send(Object.keys(args).map(function(name) {
        if (name == 'base64') return name + '=' + args[name];
        return name + '=' + soil(args[name]);
      }).join('&'));
    };

    function restartProgress() {
      if (this.status == 200) {
        gears.element.querySelector('#restartform-status').innerHTML = '';
        var container = document.createElement('div');
        container.innerHTML = [
          '<div class="restart-progress">',
            '<div class="restart-progress-info">',
              locUH.gears.send_restart + (typeof nop != 'undefined' && 'sd_cfg' in nop ? locUH.gears.warning_restart : ''),
            '</div>',
          '</div>'
        ].join('');

        document.body.appendChild(container.firstChild);

        restartUpdate();
      } else {
        gears.element.querySelector('#restartform-status').innerHTML = locUH.gears.restart_denied;
      }
    }
    gears.restartProgress = function(){restartProgress.bind({status: 200})();}

    function restartUpdate() {
      new xhr('GET', window.location.protocol == 'chrome-extension:' && window.parent && window.parent.curIP ? 'http://' + window.parent.curIP + '/' : '/', 'text/html', {
        load: iterator,
        error: iterator,
        abort: iterator,
        timeout: iterator
      }, 3000).send();

      function iterator() {
        if (this.status == 200) return window.location = window.location.protocol == 'chrome-extension:' && window.parent && window.parent.curIP ? 'http://' + window.parent.curIP + '/' : '/';

        setTimeout(restartUpdate, 1000);
      }
    }

    return [
      '<div id="restartform">',
        '<div class="form-title">' + locUH.gears.restart_device + '</div>',
        '<div class="form-option">',
          '<span id="restartform-status" class="form-option-title" style="color: red"></span>',
        '</div>',
        '<div class="form-option">',
          '<span class="form-option-title">' + locUH.gears.password + '</span>',
          '<span class="form-option-value">',
            '<input class="form-input" type="password"' + (gears.access.restart ? '' : 'disabled ') + 'id="restartform_local_0"></input>',
            '<input id="restart" class="form-button' + (gears.access.restart ? '' : ' disabled') + '" type="button" value="' + locUH.gears.restart + '"></input>',
          '</span>',
        '</div>',
      '</div>'
    ].join('')
  }
  function DisableHTTPForm(gears){
    gears.onInit.push(function(){
      if (gears.access.web) {
        gears.element.querySelector('#dishttp').addEventListener('click', gears.disable, false);
      }

    })
    gears.onLoad = [];
    gears.disable = function(){
		//окно вопроса
		gears.element.querySelector('#dishttpform-status').innerHTML = '';
        var container = document.createElement('div');
        container.innerHTML = [
          '<div class="dishttp-back">',
			'<div class="dishttp-dlg">',
				'<div>',
				  locUH.gears.dlg_question,
				'</div>',
				'<div class="dishttpform-status">',
				'</div>',
				'<div class="dishttp-dlg-buttons">',
					'<div class="dishttp-dlg-button dishttp-dlg-button-yes">' + locUH.gears.disable + '</div><div class="dishttp-dlg-button dishttp-dlg-button-no">' + locUH.gears.cancel + '</div>',
				'</div>',
			'</div>',
          '</div>'
        ].join('');

        document.body.appendChild(container.firstChild);
		document.body.querySelector('.dishttp-dlg-button-yes').addEventListener('click', gears.sendDisable, false);
		document.body.querySelector('.dishttp-dlg-button-no').addEventListener('click', function(){
			var back = document.body.querySelector('.dishttp-back');
			if (back) document.body.removeChild(back);
		}, false);
	}
	gears.sendDisable = function(){
      var soil = btoa,
          args = {
            password: gears.element.querySelector('#dishttpform_local_0').value,
            base64: true
          };

      var r = new xhr('POST', '/~http_off', 'text/plain',
        {
          load: function(){
			if (this.status == 200) {
				console.log('отключил');
				window.location.reload();
			} else {
				gears.element.querySelector('#dishttpform-status').innerHTML = locUH.gears.dishttp_denied;
				document.body.querySelector('.dishttpform-status').innerHTML = locUH.gears.dishttp_denied;
			}
		  }
        }
      );
      if (exports.requestHeaders){
        Object.keys(exports.requestHeaders).forEach(function(key) {
          r.setRequestHeader(key, exports.requestHeaders[key]);
        });
      }
      r.setRequestHeader('Content-type', 'application/x-www-form-urlencoded');
      r.send(Object.keys(args).map(function(name) {
        if (name == 'base64') return name + '=' + args[name];
        return name + '=' + soil(args[name]);
      }).join('&'));
    };

    return [
      '<div id="dishttpform">',
        '<div class="form-title">' + locUH.gears.disable_http + '</div>',
        '<div class="form-option">',
          '<span id="dishttpform-status" class="form-option-title" style="color: red"></span>',
        '</div>',
        '<div class="form-option">',
          '<span class="form-option-title">' + locUH.gears.password + '</span>',
          '<span class="form-option-value">',
            '<input class="form-input" type="password"' + (gears.access.web ? '' : 'disabled ') + 'id="dishttpform_local_0"></input>',
            '<input id="dishttp" class="form-button' + (gears.access.web ? '' : ' disabled') + '" type="button" value="' + locUH.gears.disable + '"></input>',
          '</span>',
        '</div>',
      '</div>'
    ].join('')
  }
  function LoadForm(gears){
    gears.onInit.push(function(){
      gears.element.querySelector('#load').addEventListener('click', gears.load, false)
    })
    gears.onLoad = [];
    gears.load = function(){
      gears.onLoad.forEach(function(callback){callback()});
    }
    return [
      '<div id="loadform">',
        '<div class="form-title">' + locUH.gears.external_conf + '</div>',
        '<input class="form-file" type="file" id="loadform_local_0"></input>',
        '<input id="load" class="form-button" type="button" value="' + locUH.gears.load + '"></input>',
      '</div>'
    ].join('')
  }
  function SaveForm(gears){
    gears.onInit.push(function(){
      gears.element.querySelector('#save').addEventListener('click', gears.save, false);
      if (!gears.access.xmlwr) {
        gears.element.querySelector('#saveform_device').addEventListener('click', gears.save, false);
        gears.element.querySelector('#saveform_device').checked = false;
        gears.element.querySelector('#saveform_local').checked = true;
      }
    })
    gears.onSave = [];
    gears.save = function(){
      gears.onSave.forEach(function(callback){callback()});
    }
    return [
      '<div id="saveform">',
        '<div class="form-title">' + locUH.gears.save_param + '</div>',
        '<input class="form-checkbox" type="checkbox" id="saveform_device" ' + (gears.access.xmlwr ? '' : 'disabled ') + 'checked></input>',
        '<label class="form-label ' + (gears.access.xmlwr ? '' : 'block') + '" for="saveform_device">' + locUH.gears.send_to_device + '</label><br/>',
        '<input class="form-checkbox" type="checkbox" id="saveform_local"></input>',
        '<label class="form-label" for="saveform_local">' + locUH.gears.local_save + '</label>',
        '<input id="save" class="form-button" type="button" value="' + locUH.gears.save + '"></input>',
      '</div>'
    ].join('')
  }
  function TimeForm(gears){
    gears.onInit.push(function(){
      if (gears.access.time) {
        gears.element.querySelector('#time_conf').addEventListener('click', gears.time, false);
        gears.element.querySelector('#time_loc').addEventListener('click', gears.sysTime, false);
      }
    })

    var getDT = function(){
      var dt = new Date();
      return {
        dt: dt,
        sd: dt.getFullYear() + '-' + ('0'+(dt.getMonth() + 1)).slice(-2) + '-' + ('0' + dt.getDate()).slice(-2),
        st: ('0'+dt.getHours()).slice(-2) + ':' + ('0'+dt.getMinutes()).slice(-2) + ':' + ('0'+dt.getSeconds()).slice(-2),
        utc: false
      }
    }

    gears.onTime = [];
    gears.time = function(){
      gears.onTime[0]();
    }
    gears.sysTime = function(){
      var bt = this;
      if ($(bt).hasClass('clicked')) return;
      $(bt).addClass('clicked');
      var pass = $('#timeform_local_0').val(),
          dt = getDT();

      //var url = '/~set_time?date=' + dt.sd + '&time=' + dt.st + (dt.utc ? '&utc=1' : '') + '&password=' + pass;
      var data = {
          date: dt.sd,
          time: dt.st,
          base64: true,
          password: btoa(pass)
      }
      if (dt.utc) data.utc = 1;
      $.ajax({
        url: '/~set_time',
        type: 'POST',
        timeout: 5000,
        data: data,
        contentType: 'application/x-www-form-urlencoded',
        success: function(e){
          $(bt).removeClass('clicked');
          gears.onTime[1](dt.dt);
          $('#time-status', gears.element).removeClass('red').text(locUH.gears.time_changed).addClass('green')
        },
        error: function(e){
          $(bt).removeClass('clicked');
          if(e.status == 403){
            $('#time-status', gears.element).removeClass('green').text(locUH.gears.wrong_password).addClass('red');
          } else {
						if(e.status == 423) {
							//выесняем в чем проблема
							var st = JSON.parse(e.responseText);
							if (st['Current source'] == 'GPS') {
								$('#time-status', gears.element).removeClass('green').text(locUH.gears.GPS).addClass('red');
								return;
							}
							if (st['HTTP'] == 'BLOCKED') {
								$('#time-status', gears.element).removeClass('green').text(locUH.gears.HTTPBlocked + st['Active in'] + locUH.gears.HTTPBlockedSec).addClass('red');
								return;
							}
							$('#time-status', gears.element).removeClass('green').text(locUH.gears.HTTPNotActive).addClass('red');
							//
						} else $('#time-status', gears.element).removeClass('green').text(locUH.gears.error).addClass('red');
					}
        }
      })
    }
    //if(!gears.access.time)
    return [
      '<div id="loadform">',
        '<div class="form-title">' + locUH.gears.time_conf + '</div>',
        '<div class="form-option">',
          '<span id="time-status" class="form-option-title"></span>',
        '</div>',
        '<div class="form-option">',
          '<span class="form-option-title">' + locUH.gears.password + '</span>',
          '<span class="form-option-value">',
            '<input class="form-input" type="password" ' + (gears.access.time ? '' : 'disabled ') + 'id="timeform_local_0"></input>',
          '</span>',
          '<div class="form-content">',
            '<input id="time_loc" class="form-button local-button' + (gears.access.time ? '' : ' disabled') + '" type="button" value="' + locUH.gears.set_local + '"></input>',
            '<input id="time_conf" class="form-button' + (gears.access.time ? '' : ' disabled') + '" type="button" value="' + locUH.gears.time + '"></input>',
          '</div>',
        '</div>',
      '</div>'
    ].join('')
  }
}(window, document, undefined));
